@@ -1,70 +1,89 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "debuggersourcepathmappingwidget.h"
-#include "debuggerengine.h"
 
-#include <coreplugin/variablechooser.h>
+#include "commonoptionspage.h"
+#include "debuggerengine.h"
+#include "debuggertr.h"
 
 #include <utils/buildablehelperlibrary.h>
 #include <utils/fancylineedit.h>
+#include <utils/fileutils.h>
+#include <utils/hostosinfo.h>
+#include <utils/layoutbuilder.h>
 #include <utils/pathchooser.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
-#include <utils/synchronousprocess.h>
+#include <utils/variablechooser.h>
 
-#include <QStandardItemModel>
-#include <QTreeView>
+#include <QFileDialog>
+#include <QFormLayout>
+#include <QGroupBox>
+#include <QLabel>
 #include <QLineEdit>
 #include <QPushButton>
-#include <QFormLayout>
-#include <QFileDialog>
-#include <QLabel>
+#include <QStandardItemModel>
+#include <QTreeView>
 
 using namespace Utils;
 
-// Qt's various build paths for unpatched versions.
-#if defined(Q_OS_WIN)
-static const char* qtBuildPaths[] = {
-    "Q:/qt5_workdir/w/s",
-    "C:/work/build/qt5_workdir/w/s",
-    "c:/users/qt/work/qt",
-    "c:/Users/qt/work/install"
-};
-#elif defined(Q_OS_MAC)
-static const char* qtBuildPaths[] = {};
-#else
-static const char* qtBuildPaths[] = {"/var/tmp/qt-src"};
-#endif
+namespace Debugger::Internal {
+
+class SourcePathMappingModel;
 
 enum { SourceColumn, TargetColumn, ColumnCount };
 
-namespace Debugger {
-namespace Internal {
+using Mapping = QPair<FilePath, FilePath>;
 
-typedef QPair<QString, QString> Mapping;
-typedef DebuggerSourcePathMappingWidget::SourcePathMap SourcePathMap;
+class DebuggerSourcePathMappingWidget : public QGroupBox
+{
+public:
+    DebuggerSourcePathMappingWidget();
+
+    SourcePathMap sourcePathMap() const;
+    void setSourcePathMap(const SourcePathMap &);
+
+private:
+    void slotAdd();
+    void slotAddQt();
+    void slotRemove();
+    void slotCurrentRowChanged(const QModelIndex &,const QModelIndex &);
+    void slotEditSourceFieldChanged();
+    void slotEditTargetFieldChanged();
+
+    void resizeColumns();
+    void updateEnabled();
+    QString editSourceField() const;
+    QString editTargetField() const;
+    void setEditFieldMapping(const Mapping &m);
+    int currentRow() const;
+    void setCurrentRow(int r);
+
+    SourcePathMappingModel *m_model;
+    QTreeView *m_treeView;
+    QPushButton *m_addButton;
+    QPushButton *m_addQtButton;
+    QPushButton *m_removeButton;
+    QLineEdit *m_sourceLineEdit;
+    Utils::PathChooser *m_targetChooser;
+};
+
+// Qt's various build paths for unpatched versions.
+QStringList qtBuildPaths()
+{
+    if (HostOsInfo::isWindowsHost()) {
+        return {"Q:/qt5_workdir/w/s",
+                "C:/work/build/qt5_workdir/w/s",
+                "c:/users/qt/work/qt",
+                "c:/Users/qt/work/install",
+                "/Users/qt/work/qt"};
+    } else if (HostOsInfo::isMacHost()) {
+        return { "/Users/qt/work/qt" };
+    } else {
+        return { "/home/qt/work/qt" };
+    }
+}
 
 /*!
     \class Debugger::Internal::SourcePathMappingModel
@@ -107,12 +126,12 @@ private:
 
 SourcePathMappingModel::SourcePathMappingModel(QObject *parent) :
     QStandardItemModel(0, ColumnCount, parent),
-    m_newSourcePlaceHolder(DebuggerSourcePathMappingWidget::tr("<new source>")),
-    m_newTargetPlaceHolder(DebuggerSourcePathMappingWidget::tr("<new target>"))
+    m_newSourcePlaceHolder(Tr::tr("<new source>")),
+    m_newTargetPlaceHolder(Tr::tr("<new target>"))
 {
     QStringList headers;
-    headers.append(DebuggerSourcePathMappingWidget::tr("Source path"));
-    headers.append(DebuggerSourcePathMappingWidget::tr("Target path"));
+    headers.append(Tr::tr("Source path"));
+    headers.append(Tr::tr("Target path"));
     setHorizontalHeaderLabels(headers);
 }
 
@@ -121,9 +140,9 @@ SourcePathMap SourcePathMappingModel::sourcePathMap() const
     SourcePathMap rc;
     const int rows = rowCount();
     for (int r = 0; r < rows; ++r) {
-        const QPair<QString, QString> m = mappingAt(r); // Skip placeholders.
+        const Mapping m = mappingAt(r); // Skip placeholders.
         if (!m.first.isEmpty() && !m.second.isEmpty())
-            rc.insert(m.first, m.second);
+            rc.insert(m.first.toString(), m.second.toString());
     }
     return rc;
 }
@@ -131,28 +150,28 @@ SourcePathMap SourcePathMappingModel::sourcePathMap() const
 // Check a mapping whether it still contains a placeholder.
 bool SourcePathMappingModel::isNewPlaceHolder(const Mapping &m) const
 {
-    const QLatin1Char lessThan('<');
-    const QLatin1Char greaterThan('<');
+    const QChar lessThan('<');
+    const QChar greaterThan('>');
     return m.first.isEmpty() || m.first.startsWith(lessThan)
            || m.first.endsWith(greaterThan)
-           || m.first == m_newSourcePlaceHolder
+           || m.first.toString() == m_newSourcePlaceHolder
            || m.second.isEmpty() || m.second.startsWith(lessThan)
            || m.second.endsWith(greaterThan)
-           || m.second == m_newTargetPlaceHolder;
+           || m.second.toString() == m_newTargetPlaceHolder;
 }
 
 // Return raw, unfixed mapping
 Mapping SourcePathMappingModel::rawMappingAt(int row) const
 {
-    return Mapping(item(row, SourceColumn)->text(), item(row, TargetColumn)->text());
+    return Mapping(FilePath::fromUserInput(item(row, SourceColumn)->text()),
+                   FilePath::fromUserInput(item(row, TargetColumn)->text()));
 }
 
 // Return mapping, empty if it is the place holder.
 Mapping SourcePathMappingModel::mappingAt(int row) const
 {
     const Mapping raw = rawMappingAt(row);
-    return isNewPlaceHolder(raw) ? Mapping()
-        : Mapping(QDir::cleanPath(raw.first), QDir::cleanPath(raw.second));
+    return isNewPlaceHolder(raw) ? Mapping() : Mapping(raw.first, raw.second);
 }
 
 void SourcePathMappingModel::setSourcePathMap(const SourcePathMap &m)
@@ -166,9 +185,9 @@ void SourcePathMappingModel::setSourcePathMap(const SourcePathMap &m)
 void SourcePathMappingModel::addRawMapping(const QString &source, const QString &target)
 {
     QList<QStandardItem *> items;
-    QStandardItem *sourceItem = new QStandardItem(source);
+    auto sourceItem = new QStandardItem(source);
     sourceItem->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
-    QStandardItem *targetItem = new QStandardItem(target);
+    auto targetItem = new QStandardItem(target);
     targetItem->setFlags(Qt::ItemIsEnabled|Qt::ItemIsSelectable);
     items << sourceItem << targetItem;
     appendRow(items);
@@ -197,25 +216,24 @@ void SourcePathMappingModel::setTarget(int row, const QString &t)
     Path mappings to be applied using source path substitution in GDB.
 */
 
-DebuggerSourcePathMappingWidget::DebuggerSourcePathMappingWidget(QWidget *parent) :
-    QGroupBox(parent),
+DebuggerSourcePathMappingWidget::DebuggerSourcePathMappingWidget() :
     m_model(new SourcePathMappingModel(this)),
     m_treeView(new QTreeView(this)),
-    m_addButton(new QPushButton(tr("Add"), this)),
-    m_addQtButton(new QPushButton(tr("Add Qt sources..."), this)),
-    m_removeButton(new QPushButton(tr("Remove"), this)),
+    m_addButton(new QPushButton(Tr::tr("Add"), this)),
+    m_addQtButton(new QPushButton(Tr::tr("Add Qt sources..."), this)),
+    m_removeButton(new QPushButton(Tr::tr("Remove"), this)),
     m_sourceLineEdit(new QLineEdit(this)),
     m_targetChooser(new PathChooser(this))
 {
-    setTitle(tr("Source Paths Mapping"));
-    setToolTip(tr("<p>Mappings of source file folders to "
+    setTitle(Tr::tr("Source Paths Mapping"));
+    setToolTip(Tr::tr("<p>Mappings of source file folders to "
                   "be used in the debugger can be entered here.</p>"
                   "<p>This is useful when using a copy of the source tree "
                   "at a location different from the one "
                   "at which the modules where built, for example, while "
                   "doing remote debugging.</p>"
                   "<p>If source is specified as a regular expression by starting it with an "
-                  "open parenthesis, Qt Creator matches the paths in the ELF with the "
+                  "open parenthesis, the paths in the ELF are matched with the "
                   "regular expression to automatically determine the source path.</p>"
                   "<p>Example: <b>(/home/.*/Project)/KnownSubDir -> D:\\Project</b> will "
                   "substitute ELF built by any user to your local project directory.</p>"));
@@ -232,8 +250,8 @@ DebuggerSourcePathMappingWidget::DebuggerSourcePathMappingWidget(QWidget *parent
     auto buttonLayout = new QVBoxLayout;
     buttonLayout->addWidget(m_addButton);
     buttonLayout->addWidget(m_addQtButton);
-    m_addQtButton->setVisible(sizeof(qtBuildPaths) > 0);
-    m_addQtButton->setToolTip(tr("<p>Add a mapping for Qt's source folders "
+    m_addQtButton->setVisible(!qtBuildPaths().isEmpty());
+    m_addQtButton->setToolTip(Tr::tr("<p>Add a mapping for Qt's source folders "
         "when using an unpatched version of Qt."));
     buttonLayout->addWidget(m_removeButton);
     connect(m_addButton, &QAbstractButton::clicked,
@@ -251,30 +269,30 @@ DebuggerSourcePathMappingWidget::DebuggerSourcePathMappingWidget(QWidget *parent
 
     // Edit part
     m_targetChooser->setExpectedKind(PathChooser::ExistingDirectory);
-    m_targetChooser->setHistoryCompleter(QLatin1String("Debugger.MappingTarget.History"));
+    m_targetChooser->setHistoryCompleter("Debugger.MappingTarget.History");
     connect(m_sourceLineEdit, &QLineEdit::textChanged,
             this, &DebuggerSourcePathMappingWidget::slotEditSourceFieldChanged);
-    connect(m_targetChooser, &PathChooser::pathChanged,
+    connect(m_targetChooser, &PathChooser::textChanged,
             this, &DebuggerSourcePathMappingWidget::slotEditTargetFieldChanged);
     auto editLayout = new QFormLayout;
-    const QString sourceToolTip = tr("<p>The source path contained in the "
+    const QString sourceToolTip = Tr::tr("<p>The source path contained in the "
         "debug information of the executable as reported by the debugger");
-    auto editSourceLabel = new QLabel(tr("&Source path:"));
+    auto editSourceLabel = new QLabel(Tr::tr("&Source path:"));
     editSourceLabel->setToolTip(sourceToolTip);
     m_sourceLineEdit->setToolTip(sourceToolTip);
     editSourceLabel->setBuddy(m_sourceLineEdit);
     editLayout->addRow(editSourceLabel, m_sourceLineEdit);
 
-    const QString targetToolTip = tr("<p>The actual location of the source "
+    const QString targetToolTip = Tr::tr("<p>The actual location of the source "
         "tree on the local machine");
-    auto editTargetLabel = new QLabel(tr("&Target path:"));
+    auto editTargetLabel = new QLabel(Tr::tr("&Target path:"));
     editTargetLabel->setToolTip(targetToolTip);
     editTargetLabel->setBuddy(m_targetChooser);
     m_targetChooser->setToolTip(targetToolTip);
     editLayout->addRow(editTargetLabel, m_targetChooser);
     editLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
 
-    auto chooser = new Core::VariableChooser(this);
+    auto chooser = new VariableChooser(this);
     chooser->addSupportedWidget(m_targetChooser->lineEdit());
 
     // Main layout
@@ -292,20 +310,19 @@ QString DebuggerSourcePathMappingWidget::editSourceField() const
 
 QString DebuggerSourcePathMappingWidget::editTargetField() const
 {
-    return m_targetChooser->path();
+    return m_targetChooser->unexpandedFilePath().toString();
 }
 
 void DebuggerSourcePathMappingWidget::setEditFieldMapping(const Mapping &m)
 {
-    m_sourceLineEdit->setText(m.first);
-    m_targetChooser->setPath(m.second);
+    m_sourceLineEdit->setText(m.first.toUserOutput());
+    m_targetChooser->setFilePath(m.second);
 }
 
 void DebuggerSourcePathMappingWidget::slotCurrentRowChanged
     (const QModelIndex &current, const QModelIndex &)
 {
-    setEditFieldMapping(current.isValid()
-        ? m_model->mappingAt(current.row()) : Mapping());
+    setEditFieldMapping(current.isValid() ? m_model->mappingAt(current.row()) : Mapping());
     updateEnabled();
 }
 
@@ -364,13 +381,11 @@ void DebuggerSourcePathMappingWidget::slotAdd()
 void DebuggerSourcePathMappingWidget::slotAddQt()
 {
     // Add a mapping for various Qt build locations in case of unpatched builds.
-    const QString qtSourcesPath =
-        QFileDialog::getExistingDirectory(this, tr("Qt Sources"));
+    const FilePath qtSourcesPath = FileUtils::getExistingDirectory(this, Tr::tr("Qt Sources"));
     if (qtSourcesPath.isEmpty())
         return;
-    const size_t buildPathCount = sizeof(qtBuildPaths)/sizeof(qtBuildPaths[0]);
-    for (size_t i = 0; i != buildPathCount; ++i) // use != to avoid 0<0 which triggers warning on Mac
-        m_model->addMapping(QString::fromLatin1(qtBuildPaths[i]), qtSourcesPath);
+    for (const QString &buildPath : qtBuildPaths())
+        m_model->addMapping(buildPath, qtSourcesPath.toString());
     resizeColumns();
     setCurrentRow(m_model->rowCount() - 1);
 }
@@ -400,64 +415,121 @@ void DebuggerSourcePathMappingWidget::slotEditTargetFieldChanged()
     }
 }
 
-// Find Qt installation by running qmake
-static QString findQtInstallPath(const FileName &qmakePath)
+/* Merge settings for an installed Qt (unless another setting is already in the map. */
+SourcePathMap mergePlatformQtPath(const DebuggerRunParameters &sp, const SourcePathMap &in)
 {
-    if (qmakePath.isEmpty())
-        return QString();
-    QProcess proc;
-    QStringList args;
-    args.append(QLatin1String("-query"));
-    args.append(QLatin1String("QT_INSTALL_HEADERS"));
-    proc.start(qmakePath.toString(), args);
-    if (!proc.waitForStarted()) {
-        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(qmakePath.toString()),
-           qPrintable(proc.errorString()));
-        return QString();
-    }
-    proc.closeWriteChannel();
-    if (!proc.waitForFinished() && proc.state() == QProcess::Running) {
-        SynchronousProcess::stopProcess(proc);
-        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(qmakePath.toString()));
-        return QString();
-    }
-    if (proc.exitStatus() != QProcess::NormalExit) {
-        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(qmakePath.toString()));
-        return QString();
-    }
-    const QByteArray ba = proc.readAllStandardOutput().trimmed();
-    QDir dir(QString::fromLocal8Bit(ba));
-    if (dir.exists() && dir.cdUp())
-        return dir.absolutePath();
-    return QString();
-}
+    static const QString qglobal = "qtbase/src/corelib/global/qglobal.h";
+    const FilePath sourceLocation = sp.qtSourceLocation;
+    if (!(sourceLocation / qglobal).exists())
+        return in;
 
-/* Merge settings for an installed Qt (unless another setting
- * is already in the map. */
-DebuggerSourcePathMappingWidget::SourcePathMap
-    DebuggerSourcePathMappingWidget::mergePlatformQtPath(const DebuggerRunParameters &sp,
-                                                         const SourcePathMap &in)
-{
-    const FileName qmake = BuildableHelperLibrary::findSystemQt(sp.inferior.environment);
-    // FIXME: Get this from the profile?
-    //        We could query the QtVersion for this information directly, but then we
-    //        will need to add a dependency on QtSupport to the debugger.
-    //
-    //        The profile could also get a function to extract the required information from
-    //        its information to avoid this dependency (as we do for the environment).
-    const QString qtInstallPath = findQtInstallPath(qmake);
     SourcePathMap rc = in;
-    const size_t buildPathCount = sizeof(qtBuildPaths)/sizeof(const char *);
-    if (qtInstallPath.isEmpty() || buildPathCount == 0)
-        return rc;
-
-    for (size_t i = 0; i != buildPathCount; ++i) { // use != to avoid 0<0 which triggers warning on Mac
-        const QString buildPath = QString::fromLatin1(qtBuildPaths[i]);
+    for (const QString &buildPath : qtBuildPaths()) {
         if (!rc.contains(buildPath)) // Do not overwrite user settings.
-            rc.insert(buildPath, qtInstallPath);
+            rc.insert(buildPath, sourceLocation.path());
     }
     return rc;
 }
 
-} // namespace Internal
-} // namespace Debugger
+//
+// SourcePathMapAspect
+//
+
+class SourcePathMapAspectPrivate
+{
+public:
+    QPointer<DebuggerSourcePathMappingWidget> m_widget;
+};
+
+
+SourcePathMapAspect::SourcePathMapAspect(AspectContainer *container)
+    : TypedAspect(container), d(new SourcePathMapAspectPrivate)
+{
+}
+
+SourcePathMapAspect::~SourcePathMapAspect()
+{
+    delete d;
+}
+
+void SourcePathMapAspect::fromMap(const Store &)
+{
+    QTC_CHECK(false); // This is only used via read/writeSettings
+}
+
+void SourcePathMapAspect::toMap(Store &) const
+{
+    QTC_CHECK(false);
+}
+
+bool SourcePathMapAspect::isDirty()
+{
+    guiToBuffer();
+    return m_internal != m_buffer;
+}
+
+void SourcePathMapAspect::addToLayoutImpl(Layouting::Layout &parent)
+{
+    QTC_CHECK(!d->m_widget);
+    d->m_widget = createSubWidget<DebuggerSourcePathMappingWidget>();
+    d->m_widget->setSourcePathMap(value());
+    parent.addItem(d->m_widget.data());
+}
+
+bool SourcePathMapAspect::guiToBuffer()
+{
+    const SourcePathMap old = m_buffer;
+    if (d->m_widget)
+        m_buffer = d->m_widget->sourcePathMap();
+    return m_buffer != old;
+}
+
+void SourcePathMapAspect::bufferToGui()
+{
+    if (d->m_widget)
+        d->m_widget->setSourcePathMap(m_buffer);
+}
+
+const char sourcePathMappingArrayNameC[] = "SourcePathMappings";
+const char sourcePathMappingSourceKeyC[] = "Source";
+const char sourcePathMappingTargetKeyC[] = "Target";
+
+void SourcePathMapAspect::writeSettings() const
+{
+    const SourcePathMap sourcePathMap = value();
+    QtcSettings *s = qtcSettings();
+    s->beginWriteArray(sourcePathMappingArrayNameC);
+    if (!sourcePathMap.isEmpty()) {
+        const Key sourcePathMappingSourceKey(sourcePathMappingSourceKeyC);
+        const Key sourcePathMappingTargetKey(sourcePathMappingTargetKeyC);
+        int i = 0;
+        for (auto it = sourcePathMap.constBegin(), cend = sourcePathMap.constEnd();
+             it != cend;
+             ++it, ++i) {
+            s->setArrayIndex(i);
+            s->setValue(sourcePathMappingSourceKey, it.key());
+            s->setValue(sourcePathMappingTargetKey, it.value());
+        }
+    }
+    s->endArray();
+}
+
+void SourcePathMapAspect::readSettings()
+{
+    QtcSettings *s = qtcSettings();
+    SourcePathMap sourcePathMap;
+    if (const int count = s->beginReadArray(sourcePathMappingArrayNameC)) {
+        const Key sourcePathMappingSourceKey(sourcePathMappingSourceKeyC);
+        const Key sourcePathMappingTargetKey(sourcePathMappingTargetKeyC);
+        for (int i = 0; i < count; ++i) {
+             s->setArrayIndex(i);
+             const QString key = s->value(sourcePathMappingSourceKey).toString();
+             const QString value = s->value(sourcePathMappingTargetKey).toString();
+             sourcePathMap.insert(key, value);
+        }
+    }
+    s->endArray();
+    setValue(sourcePathMap);
+}
+
+} // Debugger::Internal
