@@ -13,31 +13,122 @@
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/find/ifindsupport.h>
+#include <coreplugin/find/textfindconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 
 #include <utils/algorithm.h>
 #include <utils/fadingindicator.h>
-#include <utils/filesearch.h>
 #include <utils/futuresynchronizer.h>
+#include <utils/qtcprocess.h>
 #include <utils/qtcassert.h>
-#include <utils/stylehelper.h>
 
-#include <QDebug>
-#include <QSettings>
-#include <QHash>
-#include <QPair>
-#include <QStringListModel>
-#include <QFutureWatcher>
-#include <QPointer>
 #include <QComboBox>
+#include <QFutureWatcher>
+#include <QHash>
 #include <QLabel>
+#include <QPair>
+#include <QPointer>
+#include <QPromise>
+#include <QStringListModel>
 
 using namespace Utils;
 using namespace Core;
 
 namespace TextEditor {
+
+static std::optional<QRegularExpression> regExpFromParameters(const FileFindParameters &parameters)
+{
+    if (!(parameters.flags & FindRegularExpression))
+        return {};
+
+    const QRegularExpression::PatternOptions options = parameters.flags & FindCaseSensitively
+        ? QRegularExpression::NoPatternOption : QRegularExpression::CaseInsensitiveOption;
+    QRegularExpression regExp;
+    regExp.setPattern(parameters.text);
+    regExp.setPatternOptions(options);
+    return regExp;
+}
+
+void searchInProcessOutput(QPromise<SearchResultItems> &promise,
+                           const FileFindParameters &parameters,
+                           const ProcessSetupHandler &processSetupHandler,
+                           const ProcessOutputParser &processOutputParser)
+{
+    if (promise.isCanceled())
+        return;
+
+    QEventLoop loop;
+
+    Process process;
+    processSetupHandler(process);
+
+    const std::optional<QRegularExpression> regExp = regExpFromParameters(parameters);
+    QStringList outputBuffer;
+    // The states transition exactly in this order:
+    enum State { BelowLimit, AboveLimit, Paused, Resumed };
+    State state = BelowLimit;
+    int reportedItemsCount = 0;
+    QFuture<void> future(promise.future());
+    process.setStdOutCallback([&](const QString &output) {
+        if (promise.isCanceled()) {
+            process.close();
+            loop.quit();
+            return;
+        }
+        // The SearchResultWidget is going to pause the search anyway, so start buffering
+        // the output.
+        if (state == AboveLimit || state == Paused) {
+            outputBuffer.append(output);
+        } else {
+            const SearchResultItems items = processOutputParser(future, output, regExp);
+            if (!items.isEmpty())
+                promise.addResult(items);
+            reportedItemsCount += items.size();
+            if (state == BelowLimit && reportedItemsCount > 200000)
+                state = AboveLimit;
+        }
+    });
+    QObject::connect(&process, &Process::done, &loop, [&loop, &promise, &state] {
+        if (state == BelowLimit || state == Resumed || promise.isCanceled())
+            loop.quit();
+    });
+
+    if (promise.isCanceled())
+        return;
+
+    process.start();
+    if (process.state() == QProcess::NotRunning)
+        return;
+
+    QFutureWatcher<void> watcher;
+    QObject::connect(&watcher, &QFutureWatcherBase::canceled, &loop, [&process, &loop] {
+        process.close();
+        loop.quit();
+    });
+    QObject::connect(&watcher, &QFutureWatcherBase::suspending, &loop, [&state] { state = Paused; });
+    QObject::connect(&watcher, &QFutureWatcherBase::resumed, &loop, [&] {
+        state = Resumed;
+        for (const QString &output : outputBuffer) {
+            if (promise.isCanceled()) {
+                process.close();
+                loop.quit();
+            }
+            const SearchResultItems items = processOutputParser(future, output, regExp);
+            if (!items.isEmpty())
+                promise.addResult(items);
+        }
+        outputBuffer.clear();
+        if (process.state() == QProcess::NotRunning)
+            loop.quit();
+    });
+    watcher.setFuture(future);
+    if (promise.isCanceled())
+        return;
+    loop.exec(QEventLoop::ExcludeUserInputEvents);
+}
+
 namespace Internal {
 
 class InternalEngine : public TextEditor::SearchEngine
@@ -48,27 +139,19 @@ public:
     QString title() const override { return Tr::tr("Internal"); }
     QString toolTip() const override { return {}; }
     QWidget *widget() const override { return m_widget; }
-    QVariant parameters() const override { return {}; }
-    void readSettings(QSettings * /*settings*/) override {}
-    void writeSettings(QSettings * /*settings*/) const override {}
-    QFuture<Utils::FileSearchResultList> executeSearch(
-            const TextEditor::FileFindParameters &parameters,
-            BaseFileFind *baseFileFind) override
+    void readSettings(const Store &) override {}
+    void writeSettings(Store &) const override {}
+    SearchExecutor searchExecutor() const override
     {
-        const auto func = parameters.flags & FindRegularExpression ? Utils::findInFilesRegExp
-                                                                   : Utils::findInFiles;
-
-        return func(parameters.text,
-                    baseFileFind->files(parameters.nameFilters, parameters.exclusionFilters,
-                                        parameters.additionalParameters),
-                    textDocumentFlagsForFindFlags(parameters.flags),
-                    TextDocument::openedTextDocumentContents());
-
+        return [](const FileFindParameters &parameters) {
+            return Utils::findInFiles(parameters.text, parameters.fileContainerProvider(),
+                                      parameters.flags, TextDocument::openedTextDocumentContents());
+        };
     }
-    Core::IEditor *openEditor(const Core::SearchResultItem &/*item*/,
-                              const TextEditor::FileFindParameters &/*parameters*/) override
+
+    Utils::FindFlags supportedFindFlags() const override
     {
-        return nullptr;
+        return FindCaseSensitively | FindWholeWords | FindRegularExpression | DontFindBinaryFiles;
     }
 
 private:
@@ -91,8 +174,6 @@ public:
 class BaseFileFindPrivate
 {
 public:
-    BaseFileFindPrivate() { m_futureSynchronizer.setCancelOnWait(true); }
-
     QPointer<IFindSupport> m_currentFindSupport;
 
     Utils::FutureSynchronizer m_futureSynchronizer;
@@ -108,6 +189,7 @@ public:
     QVector<SearchEngine *> m_searchEngines;
     InternalEngine m_internalSearchEngine;
     int m_currentSearchEngineIndex = -1;
+    FilePath m_searchDir;
 };
 
 } // namespace Internal
@@ -208,34 +290,7 @@ void BaseFileFind::setCurrentSearchEngine(int index)
         return;
     d->m_currentSearchEngineIndex = index;
     emit currentSearchEngineChanged();
-}
-
-static QString displayText(const QString &line)
-{
-    QString result = line;
-    auto end = result.end();
-    for (auto it = result.begin(); it != end; ++it) {
-        if (!it->isSpace() && !it->isPrint())
-            *it = QChar('?');
-    }
-    return result;
-}
-
-static void displayResult(QFutureWatcher<FileSearchResultList> *watcher,
-                          SearchResult *search, int index)
-{
-    const FileSearchResultList results = watcher->resultAt(index);
-    QList<SearchResultItem> items;
-    for (const FileSearchResult &result : results) {
-        SearchResultItem item;
-        item.setFilePath(result.fileName);
-        item.setMainRange(result.lineNumber, result.matchStart, result.matchLength);
-        item.setLineText(displayText(result.matchingLine));
-        item.setUseTextEditorFont(true);
-        item.setUserData(result.regexpCapturedTexts);
-        items << item;
-    }
-    search->addResults(items, SearchResult::AddOrdered);
+    emit supportedFlagsChanged();
 }
 
 void BaseFileFind::runNewSearch(const QString &txt, FindFlags findFlags,
@@ -253,16 +308,20 @@ void BaseFileFind::runNewSearch(const QString &txt, FindFlags findFlags,
                 tooltip.arg(IFindFilter::descriptionForFindFlags(findFlags)),
                 txt, searchMode, SearchResultWindow::PreserveCaseEnabled,
                 QString::fromLatin1("TextEditor"));
+    setupSearch(search);
     search->setTextToReplace(txt);
     search->setSearchAgainSupported(true);
+    SearchEngine *searchEngine = currentSearchEngine();
     FileFindParameters parameters;
     parameters.text = txt;
     parameters.flags = findFlags;
     parameters.nameFilters = fileNameFilters();
     parameters.exclusionFilters = fileExclusionFilters();
-    parameters.additionalParameters = additionalParameters();
-    parameters.searchEngineParameters = currentSearchEngine()->parameters();
-    parameters.searchEngineIndex = d->m_currentSearchEngineIndex;
+    parameters.searchDir = searchDir();
+    parameters.fileContainerProvider = fileContainerProvider();
+    parameters.editorOpener = searchEngine->editorOpener();
+    parameters.searchExecutor = searchEngine->searchExecutor();
+
     search->setUserData(QVariant::fromValue(parameters));
     connect(search, &SearchResult::activated, this, [this, search](const SearchResultItem &item) {
         openEditor(search, item);
@@ -273,10 +332,6 @@ void BaseFileFind::runNewSearch(const QString &txt, FindFlags findFlags,
     connect(search, &SearchResult::searchAgainRequested, this, [this, search] {
         searchAgain(search);
     });
-    connect(this, &BaseFileFind::enabledChanged, search, &SearchResult::requestEnabledCheck);
-    connect(search, &SearchResult::requestEnabledCheck, this, [this, search] {
-        recheckEnabled(search);
-    });
 
     runSearch(search);
 }
@@ -285,29 +340,29 @@ void BaseFileFind::runSearch(SearchResult *search)
 {
     const FileFindParameters parameters = search->userData().value<FileFindParameters>();
     SearchResultWindow::instance()->popup(IOutputPane::Flags(IOutputPane::ModeSwitch|IOutputPane::WithFocus));
-    auto watcher = new QFutureWatcher<FileSearchResultList>();
+    auto watcher = new QFutureWatcher<SearchResultItems>;
     watcher->setPendingResultsLimit(1);
     // search is deleted if it is removed from search panel
     connect(search, &QObject::destroyed, watcher, &QFutureWatcherBase::cancel);
     connect(search, &SearchResult::canceled, watcher, &QFutureWatcherBase::cancel);
     connect(search, &SearchResult::paused, watcher, [watcher](bool paused) {
         if (!paused || watcher->isRunning()) // guard against pausing when the search is finished
-            watcher->setPaused(paused);
+            watcher->setSuspended(paused);
     });
     connect(watcher, &QFutureWatcherBase::resultReadyAt, search, [watcher, search](int index) {
-        displayResult(watcher, search, index);
+        search->addResults(watcher->resultAt(index), SearchResult::AddOrdered);
     });
     // auto-delete:
     connect(watcher, &QFutureWatcherBase::finished, watcher, &QObject::deleteLater);
     connect(watcher, &QFutureWatcherBase::finished, search, [watcher, search]() {
         search->finishSearch(watcher->isCanceled());
     });
-    QFuture<FileSearchResultList> future = executeSearch(parameters);
+    QFuture<SearchResultItems> future = parameters.searchExecutor(parameters);
     watcher->setFuture(future);
     d->m_futureSynchronizer.addFuture(future);
     FutureProgress *progress = ProgressManager::addTask(future,
                                                         Tr::tr("Searching"),
-                                                        Constants::TASK_SEARCH);
+                                                        Core::Constants::TASK_SEARCH);
     connect(search, &SearchResult::countChanged, progress, [progress](int c) {
         progress->setSubtitle(Tr::tr("%n found.", nullptr, c));
     });
@@ -332,16 +387,13 @@ void BaseFileFind::addSearchEngine(SearchEngine *searchEngine)
         setCurrentSearchEngine(0);
 }
 
-void BaseFileFind::doReplace(const QString &text,
-                             const QList<SearchResultItem> &items,
+void BaseFileFind::doReplace(const QString &text, const SearchResultItems &items,
                              bool preserveCase)
 {
     const FilePaths files = replaceAll(text, items, preserveCase);
     if (!files.isEmpty()) {
-        Utils::FadingIndicator::showText(ICore::dialogParent(),
-            Tr::tr("%n occurrences replaced.", nullptr, items.size()),
-            Utils::FadingIndicator::SmallText);
-        DocumentManager::notifyFilesChangedInternally(files);
+        FadingIndicator::showText(ICore::dialogParent(),
+            Tr::tr("%n occurrences replaced.", nullptr, items.size()), FadingIndicator::SmallText);
         SearchResultWindow::instance()->hide();
     }
 }
@@ -377,53 +429,75 @@ QList<QPair<QWidget *, QWidget *>> BaseFileFind::createPatternWidgets()
     syncComboWithSettings(d->m_filterCombo, d->m_filterSetting);
     QLabel *exclusionLabel = createLabel(msgExclusionPatternLabel());
     d->m_exclusionCombo = createCombo(&d->m_exclusionStrings);
-    d->m_exclusionCombo->setToolTip(msgFilePatternToolTip());
+    d->m_exclusionCombo->setToolTip(msgFilePatternToolTip(InclusionType::Excluded));
     exclusionLabel->setBuddy(d->m_exclusionCombo);
     syncComboWithSettings(d->m_exclusionCombo, d->m_exclusionSetting);
     return {{filterLabel, d->m_filterCombo}, {exclusionLabel, d->m_exclusionCombo}};
 }
 
-void BaseFileFind::writeCommonSettings(QSettings *settings)
+void BaseFileFind::setSearchDir(const FilePath &dir)
+{
+    if (dir == d->m_searchDir)
+        return;
+    d->m_searchDir = dir;
+    emit searchDirChanged(d->m_searchDir);
+}
+
+FilePath BaseFileFind::searchDir() const
+{
+    return d->m_searchDir;
+}
+
+void BaseFileFind::writeCommonSettings(
+    Store &s, const QString &defaultFilter, const QString &defaultExclusionFilter) const
 {
     const auto fromNativeSeparators = [](const QStringList &files) -> QStringList {
         return Utils::transform(files, &QDir::fromNativeSeparators);
     };
 
-    settings->setValue("filters", fromNativeSeparators(d->m_filterStrings.stringList()));
-    if (d->m_filterCombo)
-        settings->setValue("currentFilter",
-                           QDir::fromNativeSeparators(d->m_filterCombo->currentText()));
-    settings->setValue("exclusionFilters", fromNativeSeparators(d->m_exclusionStrings.stringList()));
-    if (d->m_exclusionCombo)
-        settings->setValue("currentExclusionFilter",
-                           QDir::fromNativeSeparators(d->m_exclusionCombo->currentText()));
+    const QStringList filterStrings = fromNativeSeparators(d->m_filterStrings.stringList());
+    if (filterStrings.size() != 1 || filterStrings.first() != defaultFilter)
+        s.insert("filters", filterStrings);
+    const QString currentFilter = d->m_filterCombo
+                                      ? QDir::fromNativeSeparators(d->m_filterCombo->currentText())
+                                      : d->m_filterSetting;
+    if (currentFilter != defaultFilter)
+        s.insert("currentFilter", currentFilter);
+    const QStringList exclusionFilters = fromNativeSeparators(d->m_exclusionStrings.stringList());
+    if (exclusionFilters.size() != 1 || exclusionFilters.first() != defaultExclusionFilter)
+        s.insert("exclusionFilters", exclusionFilters);
+    const QString currentExclusionFilter = d->m_exclusionCombo ? QDir::fromNativeSeparators(
+                                               d->m_exclusionCombo->currentText())
+                                                               : d->m_exclusionSetting;
+    if (currentExclusionFilter != defaultExclusionFilter)
+        s.insert("currentExclusionFilter", currentExclusionFilter);
 
     for (const SearchEngine *searchEngine : std::as_const(d->m_searchEngines))
-        searchEngine->writeSettings(settings);
-    settings->setValue("currentSearchEngineIndex", d->m_currentSearchEngineIndex);
+        searchEngine->writeSettings(s);
+    if (d->m_currentSearchEngineIndex != 0)
+        s.insert("currentSearchEngineIndex", d->m_currentSearchEngineIndex);
 }
 
-void BaseFileFind::readCommonSettings(QSettings *settings, const QString &defaultFilter,
-                                      const QString &defaultExclusionFilter)
+void BaseFileFind::readCommonSettings(
+    const Store &s, const QString &defaultFilter, const QString &defaultExclusionFilter)
 {
     const auto toNativeSeparators = [](const QStringList &files) -> QStringList {
         return Utils::transform(files, &QDir::toNativeSeparators);
     };
 
-    const QStringList filterSetting = settings->value("filters").toStringList();
+    const QStringList filterSetting = s.value("filters").toStringList();
     const QStringList filters = filterSetting.isEmpty() ? QStringList(defaultFilter)
                                                         : filterSetting;
-    const QVariant currentFilter = settings->value("currentFilter");
-    d->m_filterSetting = currentFilter.isValid() ? currentFilter.toString()
-                                                 : filters.first();
+    const QVariant currentFilter = s.value("currentFilter");
+    d->m_filterSetting = currentFilter.isValid() ? currentFilter.toString() : defaultFilter;
     d->m_filterStrings.setStringList(toNativeSeparators(filters));
     if (d->m_filterCombo)
         syncComboWithSettings(d->m_filterCombo, d->m_filterSetting);
 
-    QStringList exclusionFilters = settings->value("exclusionFilters").toStringList();
+    QStringList exclusionFilters = s.value("exclusionFilters").toStringList();
     if (!exclusionFilters.contains(defaultExclusionFilter))
         exclusionFilters << defaultExclusionFilter;
-    const QVariant currentExclusionFilter = settings->value("currentExclusionFilter");
+    const QVariant currentExclusionFilter = s.value("currentExclusionFilter");
     d->m_exclusionSetting = currentExclusionFilter.isValid() ? currentExclusionFilter.toString()
                                                           : exclusionFilters.first();
     d->m_exclusionStrings.setStringList(toNativeSeparators(exclusionFilters));
@@ -431,16 +505,16 @@ void BaseFileFind::readCommonSettings(QSettings *settings, const QString &defaul
         syncComboWithSettings(d->m_exclusionCombo, d->m_exclusionSetting);
 
     for (SearchEngine* searchEngine : std::as_const(d->m_searchEngines))
-        searchEngine->readSettings(settings);
-    const int currentSearchEngineIndex = settings->value("currentSearchEngineIndex", 0).toInt();
+        searchEngine->readSettings(s);
+    const int currentSearchEngineIndex = s.value("currentSearchEngineIndex", 0).toInt();
     syncSearchEngineCombo(currentSearchEngineIndex);
 }
 
 void BaseFileFind::openEditor(SearchResult *result, const SearchResultItem &item)
 {
     const FileFindParameters parameters = result->userData().value<FileFindParameters>();
-    IEditor *openedEditor =
-            d->m_searchEngines[parameters.searchEngineIndex]->openEditor(item, parameters);
+    IEditor *openedEditor = parameters.editorOpener ? parameters.editorOpener(item, parameters)
+                                                    : nullptr;
     if (!openedEditor)
         EditorManager::openEditorAtSearchResult(item, Id(), EditorManager::DoNotSwitchToDesignMode);
     if (d->m_currentFindSupport)
@@ -467,23 +541,22 @@ void BaseFileFind::searchAgain(SearchResult *search)
     runSearch(search);
 }
 
-void BaseFileFind::recheckEnabled(SearchResult *search)
+void BaseFileFind::setupSearch(SearchResult *search)
 {
-    if (!search)
-        return;
-    search->setSearchAgainEnabled(isEnabled());
+    connect(this, &IFindFilter::enabledChanged, search, [this, search] {
+        search->setSearchAgainEnabled(isEnabled());
+    });
 }
 
-FilePaths BaseFileFind::replaceAll(const QString &text,
-                                   const QList<SearchResultItem> &items,
+FilePaths BaseFileFind::replaceAll(const QString &text, const SearchResultItems &items,
                                    bool preserveCase)
 {
     if (items.isEmpty())
         return {};
 
-    RefactoringChanges refactoring;
+    PlainRefactoringFileFactory refactoring;
 
-    QHash<FilePath, QList<SearchResultItem> > changes;
+    QHash<FilePath, SearchResultItems> changes;
     for (const SearchResultItem &item : items)
         changes[FilePath::fromUserInput(item.path().constFirst())].append(item);
 
@@ -504,24 +577,27 @@ FilePaths BaseFileFind::replaceAll(const QString &text,
 
     for (auto it = changes.cbegin(), end = changes.cend(); it != end; ++it) {
         const FilePath filePath = it.key();
-        const QList<SearchResultItem> changeItems = it.value();
+        const SearchResultItems changeItems = it.value();
 
         ChangeSet changeSet;
         RefactoringFilePtr file = refactoring.file(filePath);
         QSet<QPair<int, int>> processed;
         for (const SearchResultItem &item : changeItems) {
             const QPair<int, int> p{item.mainRange().begin.line, item.mainRange().begin.column};
-            if (processed.contains(p))
+            if (!Utils::insert(processed, p))
                 continue;
-            processed.insert(p);
 
             QString replacement;
             if (item.userData().canConvert<QStringList>() && !item.userData().toStringList().isEmpty()) {
                 replacement = Utils::expandRegExpReplacement(text, item.userData().toStringList());
             } else if (preserveCase) {
-                const QString originalText = (item.mainRange().length(item.lineText()) == 0)
-                                                 ? item.lineText()
-                                                 : item.mainRange().mid(item.lineText());
+                Text::Range range = item.mainRange();
+                range.end.line -= range.begin.line - 1;
+                range.begin.line = 1;
+                QString originalText = item.lineText();
+                const int rangeLength = range.length(item.lineText());
+                if (rangeLength > 0)
+                    originalText = originalText.mid(range.begin.column, rangeLength);
                 replacement = Utils::matchCaseReplacement(originalText, text);
             } else {
                 replacement = text;
@@ -533,23 +609,12 @@ FilePaths BaseFileFind::replaceAll(const QString &text,
                                            item.mainRange().end.column + 1);
             changeSet.replace(start, end, replacement);
         }
-        file->setChangeSet(changeSet);
-        file->apply();
+        file->apply(changeSet);
     }
 
     return changes.keys();
 }
 
-QVariant BaseFileFind::getAdditionalParameters(SearchResult *search)
-{
-    return search->userData().value<FileFindParameters>().additionalParameters;
-}
-
-QFuture<FileSearchResultList> BaseFileFind::executeSearch(const FileFindParameters &parameters)
-{
-    return d->m_searchEngines[parameters.searchEngineIndex]->executeSearch(parameters, this);
-}
-
 namespace Internal {
 
 } // namespace Internal
